
#pragma once

#include <string>
#include <list>
#include <vector>
#include <map>

#include "CommonType.h"
#include "Observer.h"
#include "GTObject.h"

class IGTMessengerActor;
class IGTWASActor;
class IGTProfileImageActor;
class IGTProfileImage;
class IGTBuddyGroup;
class IGTBuddyUser;
class IGTLocalDBProxy;
class IGTAuthTicketProxy;
class IGTProfileImageProxy;
class INcAudioDeviceMngr;

// ActorEvent Member function rule.
// return type : bool
//  - true = continuous calling event callbakc function..
//  - false = stop calling event callback function..
//
//


////////////////////////////////////////////////////
// Messenger Actor Event Callback
class IGTMessengerActorEvent
{
public:
	virtual ~IGTMessengerActorEvent(void) { }

	virtual bool OnGTMessengerActor_UpdateState(IGTMessengerActor *pActor, GTMessengerActorState state) { return true; }
	virtual bool OnGTMessengerActor_UpdateMyStatus(IGTMessengerActor *pActor, BYTE flag, GTUserStatusType presence, NTIPTStatusType iptPrev, NTIPTStatusType iptCur) { return true; }
	virtual bool OnGTMessengerActor_UpdateMyTalkName(IGTMessengerActor *pActor, LPCWSTR strTalkName) { return true; }
	virtual bool OnGTMessengerActor_GetBuddyStatusNotify(IGTMessengerActor *pActor, std::map<WSTRING, BYTE> &m_mapBuddyInfo) { return true; }
	virtual bool OnGTMessengerActor_UpdateBuddyStatus(IGTMessengerActor *pActor, LPCWSTR strBuddyID, int nTypeBits) { return true; }
	virtual bool OnGTMessengerActor_UpdateNotBuddyUserStatus(IGTMessengerActor *pActor, LPCWSTR strBuddyID, int nUserStatus, LPCWSTR strNick, int nIPTStatus, int nFlag) { return true; }
	virtual bool OnGTMessengerActor_UpdateListBuddy(IGTMessengerActor *pActor) { return true; }
	virtual bool OnGTMessengerActor_NotifySystemMessage(IGTMessengerActor *pActor, const ChangeNoticePushData &chageNotice) { return true; }
	virtual bool OnGTMessengerActor_NotifyRecvNote(IGTMessengerActor *pActor, const ULONGLONG& lNoteId, const LPCWSTR strFromID, const LPCWSTR strFromName, const ULONG &nUnreadNoteCnt) { return true; }
	virtual bool OnGTMessengerActor_NotifyRecvUnreadNoteCnt(IGTMessengerActor *pActor, const ULONG& lUnreadCnt) { return true; }
	virtual bool OnGTMessengerActor_NotifyRecvChatInvite(IGTMessengerActor *pActor, const GT_MEMO_DATA& sData) { return true; }
	virtual bool OnGTMessengerActor_NotifyInviteRoom(IGTMessengerActor *pActor, const GT_ROOM_INVITE_NOTIFY_DATA& sData) { return true; }
	virtual bool OnGTMessengerActor_NotifyRecvUserString(IGTMessengerActor *pActor, unsigned int nUserDefinedCode, LPCWSTR strFromUserID, LPCSTR strUserString) { return true; }
	virtual bool OnGTMessengerActor_NotifyInviteBuddy(IGTMessengerActor *pActor, const GT_BUDDY_INVITE_NOTIFY_DATA& sData) { return true; }
	virtual bool OnGTMessengerActor_NotifyNewInstalledMobileUnknownUser(IGTMessengerActor *pActor, int nCacheSerial) { return true; }
	virtual bool OnGTMessengerActor_NotifyNewInstalledMobileUser(IGTMessengerActor *pActor, LPCWSTR strUserID, LPCSTR strMobileNumberHash) { return true; }
	virtual bool OnGTMessengerActor_ResponseBuddyAction(IGTMessengerActor *pActor, const GT_BUDDY_RESULT_DATA& sData) { return true; }
	virtual bool OnGTMessengerActor_ResponseUserImage(IGTMessengerActor *pActor, const vector<GT_USER_IMAGE_RESULT_DATA>& sData) { return true; }
	virtual bool OnGTMessengerActor_ResponseBuddyGroupAction(IGTMessengerActor *pActor, const GT_BUDDY_GROUP_RESULT_DATA& sData, GTBuddyAddGroupType addGroupType) { return true; }
	virtual bool OnGTMessengerActor_ResponseUpdateGroupOrder(IGTMessengerActor *pActor, trid_t trid, vector<GT_BUDDY_GROUP_DATA> &vecBuddyGroupData) { return true; }
	virtual bool OnGTMessengerActor_ResponseMobileBlockAdd(IGTMessengerActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, LPCSTR strMobileNumber) { return true; }
	virtual bool OnGTMessengerActor_ResponseMobileBlockDel(IGTMessengerActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, LPCSTR strMobileNumber) { return true; }
	virtual bool OnGTMessengerActor_ResponseSendMemo(IGTMessengerActor *pActor, const GT_MEMO_RESULT_DATA &sData) { return true; }
	virtual bool OnGTMessengerActor_NotifyIPTCallInfo(IGTMessengerActor *pActor, LPCWSTR strUserID, LPCWSTR strTelNo, LPCWSTR strCID) { return true; }
	virtual bool OnGTMessengerActor_ResponsePhoneCall(IGTMessengerActor *pActor, BYTE result, LPCWSTR strCID) { return true; }
	virtual bool OnGTMessengerActor_ResponseForwardingInfo(IGTMessengerActor *pActor, BYTE ret, BYTE act, LPCWSTR strTelNo) { return true; }
	virtual bool OnGTMessengerActor_ResponseIptResult(IGTMessengerActor *pActor, BYTE ret, UINT usIPTCMMD) { return true; }
	virtual bool OnGTMessengerActor_ResponseDndInfo(IGTMessengerActor *pActor, BYTE ret, BYTE act) { return true; }
	virtual bool OnGTMessengerActor_ResponseRefreshWorkStatus(IGTMessengerActor *pActor, trid_t trid, vector<GT_BUDDY_WORK_STATUS_DATA> &vecWorkStatusData) { return true; }
};



////////////////////////////////////////////////////
// Profile Image Actor Event Callback
class IGTProfileImageActorEvent
{
public:
	virtual ~IGTProfileImageActorEvent(void) { }

	virtual bool OnGTProfileImageActor_ResponseSetMyProfileImage(IGTProfileImageActor *pActor, const GT_PROFILE_IMAGE_RESULT_DATA& sData) { return true; }
	
	virtual bool OnGTProfileImageActor_ResponseProfileImage(IGTProfileImageActor *pActor, const GT_PROFILE_IMAGE_RESULT_DATA& sData, DWORD dwUserData1, DWORD dwUserData2, bool bCachedImage) { return true; }
};


////////////////////////////////////////////////////
// WAS Actor Event Callback
class IGTWASActorEvent
{
public:
	virtual ~IGTWASActorEvent(void) { }

	// TENANT
	virtual bool OnGTWASActor_ResponseTenantDomainList(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const GT_TENANT_DOMAIN_DATA_LIST &sList) { return true; } 
	virtual bool OnGTWASActor_ResponseGetTotalUsedCapa(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const GT_GET_TOTAL_USED_CAPA_DATA &sData) { return true; } 

	// USER
	virtual bool OnGTWASActor_ResponseCheckUserIdentify(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const GT_CHECK_USER_IDENTITY_ERROR_DATA &sInfo) { return true; } 
	virtual bool OnGTWASActor_ResponseGetUserProfile(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const GT_USER_PROFILE_DATA &sInfo) { return true; } 
	virtual bool OnGTWASActor_ResponseSetUserProfile(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const GT_USER_PROFILE_DATA &sInfo) { return true; }
    virtual bool OnGTWASActor_ResponseSetMobileUserProfile(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const GT_MOBILE_USER_PROFILE_DATA &sInfo) { return true; }
    
	// MEMBER
	virtual bool OnGTWASActor_ResponseMemberGroupList(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const GT_ORG_GROUP_DATA_LIST &list) { return true; }
	virtual bool OnGTWASActor_ResponseMemberList(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const NE_MEMBER_DATA_LIST &list) { return true; }
	virtual bool OnGTWASActor_ResponseNameSearchList(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const NE_SEARCH_MEMBER_DATA_LIST &list) { return true; }
	virtual bool OnGTWASActor_ResponseGetEmployeeInfo(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const NE_MEMBER_DATA_LIST &list, NTRequestEmpInfoType reqEmpInfoType, int nGroupID) { return true; }

	// MEMO
	virtual bool OnGTWASActor_ResponseDelMemo(IGTWASActor *pActor, const GT_MEMO_RESULT_DATA &data) { return true; } 
	virtual bool OnGTWASActor_ResponseUpdateMemo(IGTWASActor *pActor, const GT_MEMO_RESULT_DATA &data) { return true; } 
	virtual bool OnGTWASActor_ResponseSendNote(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eWasErrorCode, int nJsonErrorCode, LPCWSTR strResponseMsg) { return true; } 


	// Change Notice
	virtual bool OnGTWASActor_ResponseChangeNotice(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const CHANGE_NOTICE &chageNotice ,int nResponeCode, LPCWSTR strResponseMsg, bool bNewNotice) { return true; } 


	// CheckUp Notice

	
	virtual bool OnGTWASActor_ResponseCheckUpNotice(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const CHECKUP_NOTICE &checkUpNotice ,int nResponeCode, LPCWSTR strResponseMsg, bool bNewNotice) { return true; } 

	// NTALK NOTE
	virtual bool OnGTWASActor_ResponseUnreadNoteCount(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, int nUnreadNoteCount) { return true; } 
	virtual bool OnGTWASActor_ResponseNoteSetReadStatus(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode) { return true; } 
	virtual bool OnGTWASActor_ResponseRecvNoteInfo(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode,const NoteInfo &noteInfo, NTReqNoteInfoType reqType, UINT64 noteID) { return true; } 
	virtual bool OnGTWASActor_ResponseDeleteAttachedFile(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, bool bResult) { return true; } 
	virtual bool OnGTWASActor_ResponseDeleteNote(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, bool bResult) { return true; } 
	virtual bool OnGTWASActor_ResponseGetAttachID(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, LPCSTR szAttachID) { return true; } 
	virtual bool OnGTWASActor_ResponseGetReplyBody(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, NoteReplyBody &replyBodyInfo) { return true; } 

	virtual bool OnGTWASActor_ResponseGetTalkName(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const std::vector<NT_TALKNAME_DATA> &listTalknameData) { return true; } 
	virtual bool OnGTWASActor_ResponseGetNewRoom(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const GT_ROOM_DATA &sRoomInfo) { return true; } 
	virtual bool OnGTWASActor_ResponseGetMyRoomList(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const std::vector<NE_MYROOM_DATA> &listMyRoom) { return true; } 
	virtual bool OnGTWASActor_ResponseNoticeList(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const std::vector<NE_NOTICE_DATA> &listNoticeData) { return true; }
	virtual bool OnGTWASActor_ResponseBannerList(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const std::vector<NE_BANNER_DATA> &listBannerData) { return true; }
	virtual bool OnGTWASActor_ResponseRefreshTicket(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, LPCWSTR strSession) { return true; }
	virtual bool OnGTWASActor_ResponseGetHanatourCount(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const std::vector<NE_HANATOUR_COUNT> &listHanatourCount) { return true; }
	virtual bool OnGTWASActor_ResponseOutLinkList(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const std::vector<NE_OUTLINK_DATA> &listOutLinkData) { return true; }

	virtual bool OnGTWASActor_ResponseMailCount(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, NTNMailErrorCode eNMailError, const INT64 ulPrevMailSN, const INT64 ulMailCountData) { return true; }
	virtual bool OnGTWASActor_ResponseMailPreview(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, LPCWSTR strSubject, LPCWSTR strContent, LPCWSTR strSenderName, LPCWSTR strSenderID) { return true; }

	virtual bool OnGTWASActor_ResponseCallHistory(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const std::vector<NE_CALL_DATA> &list) { return true; }
	virtual bool OnGTWASActor_ResponseSetCallInfo(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, int nCallSeq) { return true; }
	virtual bool OnGTWASActor_ResponseSearchCallHistory(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const std::vector<NE_CALL_DATA> &list) { return true; }
	virtual bool OnGTWASActor_ResponseSetCallMemo(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, int nCallSeq) { return true; }
	virtual bool OnGTWASActor_ResponseSMS(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const NE_SMS_DATA &smsCount) { return true; }
	virtual bool OnGTWASActor_ResponseSetLanguage(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const NE_LANG_DATA &langData) { return true; }
	virtual bool OnGTWASActor_ResponseGetLanguage(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const NE_LANG_DATA &langData) { return true; }
	virtual bool OnGTWASActor_ResponseGetMessages(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, const std::vector<NE_MESSAGE_DATA> &listMessage) { return true; } 
};


template <class T>
class IGTEventActor : public CGTObserved<T>
{
public:
	IGTEventActor() { }
	virtual ~IGTEventActor() { }
};

class IGTMessengerActor : public IGTObject, public IGTEventActor<IGTMessengerActorEvent>
{
public:
	virtual ~IGTMessengerActor(){}
	
	// LOGIN
	virtual bool RequestLogin(LPCWSTR tcsUserID, LPCWSTR tcsPassword, LPCWSTR tcsLangCode, LPCWSTR tcsTenantName, bool bShowOfflineLogin = false, LPCWSTR tcsLoginCode = _T(""), LPCWSTR tcsTicketType = _T(""),
							  GTOSType eOSType = osWindows, USHORT usCountryCode = 82, LPCSTR strMobileNumber = "", LPCSTR strMobileToken = "", GTMobileOverwriteDataType eOverwriteDataType = mobileOverwriteData_Check) = 0;
	virtual bool RequestLogout(GTLogoutReasonType logout_type = lortByUserCommand) = 0;
	virtual bool RequestRelogin(void) = 0;
	virtual void DisconnectLoginServer(void) = 0;
	virtual GTMessengerActorState GetActorState(void) = 0;
	virtual GTLogoutReasonType GetLogoutReasonType(void) = 0;
	virtual void GetLoginErrorData(GT_LOGIN_ERROR_DATA &data) = 0;
	
	// MY INFO
	virtual bool IsValidLoginState(void) = 0;
	virtual bool IsEverLoginSuccess(void) = 0;
	virtual bool IsFirstLogin(void) = 0;
	virtual GTUserStatusType GetMyStatus(void) = 0;
	virtual NTIPTStatusType GetMyIptStatus(void) = 0;
	virtual void GetMyUserInfo(GT_MY_USER_DATA &data) = 0;
	virtual void GetUserLevels(NE_LEVEL_DATA_LIST &sLevels) = 0;
	virtual LPCWSTR GetUserID(void) = 0;
	virtual LPCWSTR GetTalkName(void) = 0;
	virtual LPCWSTR GetUserPassword(void) = 0;
	virtual LPCSTR GetMobilePhoneNumber(void) = 0;
	virtual void GetMyUserProfile(GT_USER_PROFILE_DATA &sData) = 0;
	virtual void GetMyMobileUserProfile(GT_MOBILE_USER_PROFILE_DATA &sData) = 0;
	
	virtual void RequestSetMyStatus(GTUserStatusType usertype, bool bForceMode = false, const char *pCustomData = NULL, int nCustomDataSize = 0, bool bCustomDataDBSave = false) = 0;
	virtual void RequestUserImageList(LPCTSTR str) = 0;
	// HEALTH CHECK
	virtual void RequestRefreshTicket(void) = 0;
	virtual void RequestSendPing(void) = 0;
	virtual bool RequestPing(void) = 0;
	
	virtual LPCSTR GetMessengerTicketType(void) = 0;
	virtual LPCSTR GetMessengerTicket(void) = 0;
	virtual LPCSTR GetNewMessengerTicket(void) = 0;
	
	virtual void SetMessengerTicket(LPCSTR strTicket) = 0;
	virtual void SetMessengerTicketType(LPCSTR strType) = 0;

    virtual unsigned int GetLaunchingResult(void) = 0;
    virtual LPCSTR GetLaunchingInfoURL(void) = 0;

	// BUDDY INFO
	virtual bool FindBuddy(LPCWSTR pszBuddyID, BYTE nType = btAll) = 0;
	virtual bool IsBlockBuddy(LPCWSTR pszBuddyID) = 0;
	virtual int GetDefaultGroupID(void) = 0;
	virtual size_t GetBuddyCount(void ) = 0;
	virtual size_t GetBuddyCount(int nGroupID) = 0;
	virtual size_t GetBuddyGroupCount(void ) = 0;
	virtual const IGTBuddyUser* GetBuddyInfoFromIndex(size_t nIndex) = 0;
	virtual const IGTBuddyUser* GetBuddyInfoFromIndex(int nGroupID, size_t nIndex) = 0;
	virtual const IGTBuddyUser* GetBuddyInfo(int nBuddyID) = 0;
	virtual const IGTBuddyUser* GetBuddyInfo(LPCWSTR pszBuddyID) = 0;
	virtual const IGTBuddyUser* GetBuddyInfo(LPCWSTR pszBuddyID, int nGroupID) = 0;
    virtual const IGTBuddyUser* GetBuddyInfoFromPersist(LPCWSTR pszBuddyID) = 0;

    //call GetBuddyInfo, if return NULL, then call GetBuddyInfoFromPersist
    virtual const IGTBuddyUser* GetAvailbeBuddyInfo(LPCWSTR pszBuddyID) = 0;
    virtual const NE_MEMBER_DATA* GetEmployeeInfo(LPCWSTR pszBuddyID) = 0;

	// including blocked users who are not even in friend relationship
	virtual size_t GetBlockUserList(GT_BUDDY_DATA_LIST &list) = 0;

	// BUDDY ONLINE STATUS
	virtual void RequestRegiFavoriteToPOI(vector<wstring> &vecUsers, USHORT nUserGrp) = 0;
	virtual void RequestUregFavoriteFromPOI(USHORT nUserGrp) = 0;
	virtual void RequestUregFavoriteFromPOI(vector<wstring> &vecUsers, USHORT nUserGrp) = 0;
	virtual void RequestGetBuddyStatus(std::vector<wstring> &vecUsers) = 0;
	
	// GROUP INFO
	virtual const IGTBuddyGroup* GetBuddyGroupInfoFromIndex(size_t nIndex) = 0;
	virtual const IGTBuddyGroup* GetBuddyGroupInfo(int nGroupID) = 0;
	virtual const IGTBuddyGroup* GetBuddyGroupInfo(LPCWSTR strGroupName) = 0;
	
	// BUDDY ACTION (Import/Add/Del/Move/Block)
	virtual bool RequestImportBuddy(std::vector<IMPORT_GROUP> &vecImportBuddy, trid_t *pTrid = NULL) = 0;
	virtual bool RequestAddBuddy(int nGroupID, std::vector<wstring> &vecBuddyID, LPCWSTR strInviteMsg, NTAddBuddyType addBuddyType = abtAddBuddy, trid_t *pTrid = NULL) = 0;
	virtual bool RequestDelBuddy(std::vector<int> &vecBuddyID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestMoveBuddy(std::vector<int> &vecBuddyID, int nNewGrpID, trid_t *pTrid) = 0;
	virtual bool RequestSearchBuddy(LPCWSTR strBuddyID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestBlockBuddy(bool bBlock, LPCWSTR strBuddyID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestAcceptBuddy(LPCWSTR strBuddyID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestRejectBuddy(LPCWSTR strBuddyID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestRefreshWorkStatus(trid_t *pTrid = NULL) = 0;//  մϴ.
	
	// GROUP ACTION (Add/Del/Update)
	virtual bool RequestAddBuddyGroup(LPCWSTR strGroupName, GTBuddyAddGroupType addGroupType,int nSortOrder = 0, trid_t *pTrid = NULL) = 0;
	virtual bool RequestDelBuddyGroup(int nGroupID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestUpdateBuddyGroup(int nGroupID, const GT_BUDDY_GROUP_UPDATABLE_DATA &sInfo, trid_t *pTrid = NULL) = 0;
	virtual bool RequestUpdateBuddyGroupSortOrder(std::vector<GT_BUDDY_GROUP_DATA> &vecGroupData, trid_t *pTrid = NULL) = 0;
	
	// USER DEFINED MESSAGE 
	virtual bool RequestSendUserString(LPCWSTR pszUserID, unsigned int nUserDefinedCommand, LPCWSTR pszUserString) = 0;
	virtual bool RequestSendUserBuffer(LPCWSTR pszUserID, unsigned int nUserDefinedCommand, const char *pBuffer, int nBufferSize) = 0;
	
	// MEMO
	virtual bool RequestSendMemo(LPCWSTR strTargetUserID, LPCWSTR strSubject, LPCWSTR strContents, trid_t *pTrid = NULL) = 0;
	virtual bool RequestSendMemoEx2(std::vector<LPCWSTR> &vecTargetUserID, LPCWSTR strContents, trid_t *pTrid = NULL) = 0;

	// AUTH
	virtual size_t GetAuthDomainCount(void) = 0;
	virtual LPCWSTR GetAuthDomainFromIndex(size_t nIndex) = 0;
	
	
	// GET ACTOR/MANAGER 
	virtual IGTWASActor *GetWASActor(void) = 0;
	virtual IGTProfileImageActor *GetProfileImageActor(void) = 0;
	
	// ETC
	virtual bool IsMobileLogin(void) = 0;
	virtual bool IsAdult(void) = 0;
	virtual bool GetURLPack(GT_URL_PACK_DATA &sPack) = 0;
	
	virtual bool GetSpecInfo(GT_API_SPEC_DATA &sSpecInfo) = 0;
	virtual bool GetFAQList(GT_FAQ_DATA_LIST &sFAQList) = 0;
    //CategoryList
	virtual int GetCategoryCacheSerial(void) = 0;
    
    // CACHE SERIAL
	virtual int GetActiveSerialRoomMetaData(void) = 0;
	virtual int GetActiveSerialGroup(void) = 0;
	virtual int GetActiveSerialBuddy(void) = 0;
	virtual int GetActiveSerialBuddyAddressBook(void) = 0;
	virtual int GetActiveSerialInviteBuddy(void) = 0;
	virtual int GetActiveSerialSpecInfo(void) = 0;
	virtual int GetActiveSerialMobileAddressBook(void) = 0;	
	virtual int GetActiveSerialRelayServer(void) = 0;
    virtual GTMobileAuthNumberStatusCheckResultMaskType GetMobileAuthNumberStatusCheckResultMaskType(void) = 0;

    // Proxy
	virtual void SetAuthTicketProxy(IGTAuthTicketProxy *proxy) = 0;	

	// IPT
	virtual bool RequestPhoneCall(LPCWSTR strUserID, LPCWSTR strPhoneNo, trid_t *pTrid = NULL) = 0;
	virtual bool RequestAnswerCall(LPCWSTR strUserID, LPCWSTR strCID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestHangUp(LPCWSTR strUserID, LPCWSTR strCID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestCallForwarding(LPCWSTR strUserID, BYTE nMode, LPCWSTR strPhoneNo, trid_t *pTrid = NULL) = 0;
	virtual bool RequestCallForwardingInfo(LPCWSTR strUserID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestCallTransfer(LPCWSTR strUserID, LPCWSTR strTargetID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestCallPickUp(LPCWSTR strUserID, LPCWSTR strTargetID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestSetDND(LPCWSTR strUserID, BYTE nMode, trid_t *pTrid = NULL) = 0;
	virtual bool RequestPhoneStatus(LPCWSTR strUserID, std::vector<LPCWSTR> &vecUserID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestConferenceCall(LPCWSTR strUserID, LPCWSTR strPhoneNo, trid_t *pTrid = NULL) = 0;
	virtual bool RequestHoldCall(LPCWSTR strUserID, LPCWSTR strCID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestResumeCall(LPCWSTR strUserID, LPCWSTR strCID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestDndStatusInfo(LPCWSTR strUserID, trid_t *pTrid = NULL) = 0;

	virtual IGTBuddyGroup* GetVirtualBuddyGroup(GT_BUDDY_GROUP_DATA data) = 0;
	virtual IGTBuddyUser*  GetVirtualBuddyUser() = 0;	
};

class IGTProfileImageActor : public IGTObject, public IGTEventActor<IGTProfileImageActorEvent>
{
public:
	virtual ~IGTProfileImageActor(){}
	
    virtual void SetProfileImageProxy(IGTProfileImageProxy *proxy) = 0;
    virtual GTProfileImageCheckResult GetUserProfileImagePath(LPCWSTR strUserID, GTProfileImageSizeType eSizeType, size_t &nBufferSize/*OUT*/, char *szFilePath /*OUT*/) = 0;
    
    virtual bool RequestSetMyProfileImage(LPCWSTR pszFileName, trid_t *pTrid = NULL) = 0;

#ifdef __USE_PROFILE_IMG_BY_P2P__
	virtual bool RequestProfileImageByP2P(LPCWSTR strUserID, GTProfileImageSizeType eSizeType = profileOriginalSize, DWORD dwUserData1 = 0, DWORD dwUserData2 = 0, bool bForceMode = false, trid_t *pTrid = NULL) = 0;
#endif

	virtual bool RequestProfileImage(LPCWSTR strUserID, GTProfileImageSizeType eSizeType = profileOriginalSize, DWORD dwUserData1 = 0, DWORD dwUserData2 = 0, bool bForceMode = false, trid_t *pTrid = NULL) = 0;
    
	virtual void StopSetMyProfileImage(void) = 0;
};

class IGTWASActor : public IGTObject, public IGTEventActor<IGTWASActorEvent>
{
public:
	virtual ~IGTWASActor(){}
	
public:
	virtual bool CancelRequest(trid_t trid) = 0;

	virtual bool RequestSendNote(NT_NOTE_SEND_DATA &sSendNoteData, trid_t *pTrid = NULL) = 0;
	virtual bool RequestUnreadNoteCount( trid_t *pTrid = NULL) = 0;
	virtual bool RequestNoteSetReadStatus( UINT64 nNoteID, LPCSTR strLanguage, trid_t *pTrid = NULL) = 0;
	virtual bool RequestGetRecvNoteInfo( UINT64 nNoteID, NTReqNoteInfoType reqType, LPCSTR strLanguage, bool bIncludeToList, trid_t *pTrid = NULL) = 0;
	virtual bool RequestDelAttachedFile( LPCSTR strAttachID, int nIndex, trid_t *pTrid = NULL) = 0;
	virtual bool RequestDeleteNote( UINT64 nNoteID, LPCSTR strLanguage, trid_t *pTrid = NULL) = 0;
	virtual bool RequestGetAttachID( trid_t *pTrid = NULL) = 0;
	virtual bool RequestGetReplyBody( LPCWSTR strWriteMode, UINT64 nNoteID, LPCSTR strLanguage, trid_t *pTrid = NULL) = 0;

	virtual bool RequestGetUserProfile(LPCWSTR strUserID, trid_t *pTrid = NULL) = 0;
	virtual bool RequestSetMyUserProfile(const GT_USER_PROFILE_DATA &sInfo, trid_t *pTrid = NULL) = 0;

	virtual bool RequestChangeNotice(LPCWSTR strURL, LPCWSTR strCorp, trid_t *pTrid = NULL) = 0;

	virtual bool RequestCheckUpNotice(LPCWSTR strURL, trid_t *pTrid = NULL) = 0;


	virtual const FILE_UPLOAD_RESULT_DATA* RequestUploadFileParsing(std::string &strJson) = 0;
    // Room Info
	virtual bool RequestGetNewRoom(LPCTSTR pszCid, LPCTSTR pszCName, LPCTSTR pszBid, LPCTSTR pszBName, trid_t *pTrid = NULL) = 0;
	virtual bool RequestGetMyRoomList(LPCWSTR strLangCode, trid_t *pTrid = NULL) = 0;
	virtual bool RequestGetMessages(int nRoomId, int nJoinSeq, int nFromMsgNum, int nToMsgNum, int nSendSuccess, trid_t *pTrid = NULL) = 0;
	
	virtual bool RequestGetTalkName(LPCTSTR strEmpIDs, trid_t *pTrid = NULL) = 0;

	virtual LPCWSTR GetLastErrorString(trid_t trid) = 0;
	
	virtual bool GetPenaltyResultInfo(trid_t trid, GT_PENALTY_RESULT_DATA &sPenaltyResult) = 0;
	virtual bool GetCacheResultInfo(trid_t trid, GT_CACHE_SERIAL_DATA &sResult) = 0;
    virtual bool GetContentProgressInfo(trid_t trid, size_t &totalSize, size_t &currentSize) = 0;

	// Member
	virtual bool RequestMemberGroupList(int nGroupType, LPCWSTR CustcoID, LPCWSTR strTenant, LPCWSTR strLangCode, trid_t *pTrid = NULL) = 0;
	virtual bool RequestMemberList(int nGroupType, LPCWSTR strParentGrpID, LPCWSTR strGrpID, LPCWSTR strLangCode, trid_t *pTrid = NULL) = 0;
	virtual bool RequestNameSearchList(const NE_SEARCH_MEMBER_PARAM& searchParam, trid_t *pTrid = NULL) = 0;
	virtual bool RequestEmplyeeInfo(LPCWSTR strEmpIDs, LPCWSTR strLangCode, NTRequestEmpInfoType reqEmpInfoType = reitSingleTrans, trid_t *pTrid = NULL, int nGroupID = -1 )=0;

	// Tenant
	virtual bool RequestTenantDomainList(LPCWSTR strTenant, trid_t *pTrid = NULL) = 0;
	virtual bool RequestGetTotalUsedCapa(trid_t *pTrid = NULL) = 0;

	virtual bool RequestListNotice(trid_t *pTrid = NULL) = 0;
	virtual bool RequestListBanner(trid_t *pTrid = NULL) = 0;
	virtual bool RequestGetHanatourCount(trid_t *pTrid = NULL) = 0;
	virtual bool RequestListOutLink(trid_t *pTrid = NULL) = 0;

	//mail
	virtual bool RequestMailCount(trid_t *pTrid = NULL) = 0;
	virtual bool RequestMailPreview(ULONGLONG ulMailSerial, trid_t *pTrid = NULL) = 0;

	virtual bool RequestSMS(LPCWSTR strContent, LPCWSTR strToTelNo, LPCWSTR strFromTelNo, trid_t *pTrid = NULL)=0;
	virtual bool RequestSetLanguage(LPCWSTR strLanguage, trid_t *pTrid = NULL)=0;
	virtual bool RequestGetLanguage(trid_t *pTrid = NULL)=0;

	// IPT
	virtual bool RequestCallHistory(NTCallType eCallType, LPCWSTR strLangCode, trid_t *pTrid = NULL) = 0;
	virtual bool RequestSetCallInfo(NE_CALL_DATA &sCallData, trid_t *pTrid = NULL) = 0;
	virtual bool RequestSearchCallHistory(NTCallType eCallType, LPCWSTR strSearchWord, LPCWSTR strLangCode, trid_t *pTrid = NULL) = 0;
	virtual	bool RequestSetCallMemo(int nCallSeq, LPCWSTR strMemo, trid_t *pTrid = NULL) = 0;

	virtual void SetWASServerAddress(const WSTRING& strServerAddr) = 0;
	virtual LPCWSTR GetWASServerAddress() = 0;
};

class IGTBuddyUser : public IGTObject
{
public:
	virtual ~IGTBuddyUser() { }
	
	virtual LPCWSTR GetUserID(void) const = 0;
	virtual LPCWSTR GetTalkName(void) const = 0;
	virtual LPCWSTR GetAlias(void) const = 0;
	virtual LPCWSTR GetName(void) const = 0;
	virtual LPCWSTR GetDisplayName(void) const = 0;
	virtual GTUserStatusType GetStatus(void) const = 0;
	virtual NTIPTStatusType GetIPTStatus(void) const = 0;
	virtual int GetID(void) const = 0;
	virtual int GetGroupID(void) const = 0;
	virtual BYTE GetType(void) const = 0;
    virtual DWORD GetProfileImageCRC(void) const = 0;
	virtual bool IsOnline(void) const = 0;
	virtual bool IsBuddy(void) const = 0;
	//virtual bool IsFakeBuddy(void) const = 0;
	virtual bool IsBlock(void) const = 0;
	virtual bool IsWait(void) const = 0;
	virtual GTOSType GetOSType(void) const = 0;
	virtual bool IsMobileUser(void) const = 0;
	virtual void GetUpdatableData(GT_BUDDY_UPDATABLE_DATA &sData) const = 0;
	virtual const char* GetCustomData(void) const = 0;
	virtual int GetCustomDataSize(void) const = 0;
};

static const int RECENT_CONTACT_GROUP_ID = -1;
static const int RECENT_CONTACT_GROUP_SORTORDER = -1;

class IGTBuddyGroup : public IGTObject
{
public:
	virtual ~IGTBuddyGroup() { }
	
	virtual int GetID(void) const = 0;
	virtual LPCWSTR GetName(void) const = 0;
	virtual int GetSortOrder(void) const = 0;
	virtual bool IsDefaultGroup(void) const = 0;
	virtual void GetUpdatableData(GT_BUDDY_GROUP_UPDATABLE_DATA &sData) const = 0;
};

class IGTProfileImageProxy : public IGTObject
{
public:
    virtual unsigned int GetCurrentImageCRC(LPCWSTR strUserId) = 0;

    virtual GTProfileImageSizeType GetImageRecvedTypeBits(LPCWSTR strUserId) = 0;

    virtual bool SetCurrentImageCRC(LPCWSTR strUserId, unsigned int dwCRC) = 0;

    virtual bool SetImageRecvedTypeBits(LPCWSTR strUserId, GTProfileImageSizeType typeBits) = 0;
};

class IGTAuthTicketProxy : public IGTObject
{
public:
	virtual bool GetAuthTicket(char *szTicket, int nSize) = 0;
};

class IGTLocalDBProxy : public IGTObject
{
public:
	virtual ~IGTLocalDBProxy() { }

	/* <Buddy Group> */
	virtual bool SaveBuddyGroupSerial(int nCacheSerial, bool bForceSaveMode = false) = 0;
	virtual int LoadBuddyGroupSerial(void) = 0;	
	virtual int GetBuddyGroupDataCount(void) = 0;
	virtual bool DeleteBuddyGroupData(int nGroupID) = 0;
	virtual bool SaveBuddyGroupData(const GT_BUDDY_GROUP_DATA &data) = 0;
	virtual bool SaveBuddyGroupDataList(const GT_BUDDY_GROUP_DATA_LIST &listGroup) = 0;
	virtual bool LoadBuddyGroupDataList(GT_BUDDY_GROUP_DATA_LIST &result) = 0;
	virtual int LoadDefaultGroupID(void) = 0;
	virtual bool SaveDefaultGroupID(int nGroupID) = 0;	
	
	/* <Buddy AddressBook> */
	virtual bool SaveBuddyAddressBookSerial(int nCacheSerial, bool bForceSaveMode = false) = 0;
	virtual int LoadBuddyAddressBookSerial(void) = 0;	
	virtual int GetBuddyAddressBookDataCount(void) = 0;
	virtual bool DeleteBuddyAddressBookData(LPCWSTR strUserID) = 0;
	virtual bool SaveBuddyAddressBookData(const GT_BUDDY_DATA &data) = 0;
	virtual bool SaveBuddyAddressBookDataList(const GT_BUDDY_DATA_LIST &listBuddy) = 0;
	virtual bool LoadBuddyAddressBookDataList(GT_BUDDY_DATA_LIST &result) = 0;
	
	/* <Invite Buddy> */
	virtual bool SaveInviteBuddySerial(int nCacheSerial, bool bForceSaveMode = false) = 0;
	virtual int LoadInviteBuddySerial(void) = 0;
	virtual int GetInviteBuddyDataCount(void) = 0;
	virtual bool SaveInviteBuddyDataList(const GT_BUDDY_INVITE_NOTIFY_DATA_LIST &listInvite) = 0;
	virtual bool LoadInviteBuddyDataList(GT_BUDDY_INVITE_NOTIFY_DATA_LIST &result) = 0;		
	
	/* <Spec Data> */
	virtual bool SaveSpecSerial(int nCacheSerial, bool bForceSaveMode = false) = 0;
	virtual int LoadSpecSerial(void) = 0;		
	virtual bool SaveSpecData(const GT_API_SPEC_DATA &data) = 0;
	virtual bool LoadSpecData(GT_API_SPEC_DATA &result) = 0;
	
	/* <Mobile Block Data> */
	virtual bool SaveMobileBlockSerial(int nCacheSerial, bool bForceSaveMode = false) = 0;
	virtual int LoadMobileBlockSerial(void) = 0;	
	virtual int GetMobileBlockDataCount(void) = 0;
	virtual bool DeleteMobileBlockData(LPCSTR strMobileNumber) = 0;
	virtual bool SaveMobileBlockData(const GT_MOBILE_BLOCK_NUMBER_DATA &data) = 0;
	virtual bool SaveMobileBlockDataList(const GT_MOBILE_BLOCK_NUMBER_DATA_LIST &listMobileBlock) = 0;
	virtual bool LoadMobileBlockDataList(GT_MOBILE_BLOCK_NUMBER_DATA_LIST &result) = 0;
};

// EOF
